#!/usr/bin/perl -w
# Copyright (C) 2003 Andrew Birkett

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

##
# www.nobugs.org image gallery script (andy@nobugs.org)

# This script is quite "robust", meaning that you just need to run it
# and it'll do the Right Thing.  Eg. the first time you run it it'll
# generate a skeleton "spec.txt" where you can add image descriptions.
# Then you just rerun it again and it'll build the gallery.  If
# something is wrong, it'll propose a solution.  I like this style of
# self-correcting helpful perl script .. there's no point in wasting
# braincells on this stuff - Andrew

# Configuration section
##

my $IMAGES_PER_ROW = 3;
my $big_image_geometry = "500x400>";  # '>' means don't exapand small images
my $thumbnail_geometry = "150x150";

##
# Configuration ends

use strict;
use CGI;
use Getopt::Long;

# Global state, updated by parse_spec_file() and used by the rest of the program
my (%short, %long, @images, $date, $title);

# Command line processing
my $opt_output_dir = "output";
my $opt_indexonly = 0;
my $opt_help = 0;

GetOptions("output-directory=s" => \$opt_output_dir, 
	   "index-only" => \$opt_indexonly,
	   "help" => \$opt_help);

if ($opt_help) {
    print <<"EOF";
gallery.pl - Builds an image gallery from a spec.txt file

Options:

  --output-directory path  Write the gallery to this directory ("output")
  --index-only             Rebuild index only, not images.
  --help                   Show this message
EOF
    exit;
}

sub is_image($)
{
    my ($file) = @_;
    m|\.jpg|i or m|\.jpeg|i or m|\.gif|i or m|\.png|i;
}

sub usage()
{
    opendir DIR, ".";
    my @image_files = sort grep { is_image($_) } (readdir(DIR));

    if (@image_files == 0) {
	print STDERR "Oops! There are no jpeg/gif/png files in this directory.  You need to\n";
	print STDERR "change into the directory which contains your image files and rerun\n";
	print STDERR "this script.\n";
	exit(1);
    }

    open FILE, ">spec.txt";
    print FILE "Title=\n";
    print FILE "Date=", scalar(localtime), "\n";

    foreach (@image_files) {
	print FILE "File=$_\n";
	print FILE "Short=A few words\n";
	print FILE "Write a description here - blank lines seperate paragraphs\n";
    }
    close FILE;

    print STDERR "I have written out a skeleton \"spec.txt\" for you. Please edit\n";
    print STDERR "the file and rerun this script to generate the gallery.\n";

    exit(0);
}

sub basename($)
{
    my ($filename) = @_;
    $filename =~ m/(.*)\.[^\.]*$/;
    $1;
}

sub print_thumbnail_info($)
{
    my ($image) = @_;
    my $base = basename($image);
    my $link = $base . ".html";
    print INDEX "<table border=3 bgcolor=\"#a0ffff\">\n";

    # Image link row
    print INDEX "<tr><td valign=center align=center bgcolor=\"#000000\">\n";
    print INDEX "<a href=\"$link\">\n";
    print INDEX "<img src=\"thumb-$image\" alt=\"$_\">\n";
    print INDEX "</a>\n";
    print INDEX "</td></tr>\n";

    # Short description row
    print INDEX "<tr><td align = center>\n";
    print INDEX "<a href=\"$link\">\n";
    print INDEX "$short{$image}\n";
    print INDEX "</a>\n";
    print INDEX "</td></tr>\n";

    print INDEX "</table>\n";
}


sub build_index
{
    my $cgi = new CGI({});

    open INDEX, ">$opt_output_dir/index.html";
    print INDEX $cgi->start_html(-title=>$title,
				 -meta=>{'keywords' => "$title image gallery"}
				 );
    print INDEX $cgi->h1($title);
    
    # Now the table containing thumbnails
    my $count = 0;
    print INDEX "<table>\n";
    print INDEX " <tr>\n";
    foreach (@images) {
	print INDEX "   <td align=center valign=center>\n";
	print_thumbnail_info($_);
	print INDEX "   </td>\n";
	$count++;
	if ($count == $IMAGES_PER_ROW) {
	    $count = 0;
	    print INDEX " </tr>\n";
	    print INDEX " <tr>\n";
	}
    }
    print INDEX " </tr>\n";
    print INDEX "</table>\n";
    print INDEX '<p><i>Produced by the nobugs.org <a href="http://www.nobugs.org/developer/gallery/index.html">gallery generator</a> ';
    
    if ($date) {
	print INDEX " on $date.";
    }
    print INDEX "</i></p>\n";

    print INDEX $cgi->end_html;
    close INDEX;
}

sub build_pages
{
    # Generate thumbnails
    foreach (@images) {
	print "Resizing main image for $_\n";
	`convert -quality 90 -geometry \"$big_image_geometry\" \"$_\" \"$opt_output_dir/$_\"`;
	print "Generating thumbnail for $_\n";
	`convert -quality 90 -geometry \"$thumbnail_geometry\" \"$_\" \"$opt_output_dir/thumb-$_\"`;
    }

    my $cgi = new CGI({});

    # Generate the image pages
    my $i;
    for ($i=0; $i < $#images + 1; $i++) {
	my $file = $images[$i];
	my $base = basename($file);
	open IMAGE, ">$opt_output_dir/$base.html";
	print IMAGE $cgi->start_html(-title=>$short{$file});
	print IMAGE $cgi->h1($short{$file});
	
	print IMAGE "<table border=0>\n";
	print IMAGE  "<tr>\n";
	
	print IMAGE "<td valign=center align=center>\n";
	print IMAGE "<img src=\"", $file, "\" alt=\"$file\">\n";
	print IMAGE "</td>\n";

	print IMAGE "<td valign=center align=center>\n";
	print IMAGE $long{$file};
	print IMAGE "<p>\n";
    
	if ($i < $#images) {
	    my $next_page = basename($images[$i + 1]) . ".html";
	    print IMAGE "<a href=\"$next_page\">next</a>";
	} else {
	    print IMAGE "(the end)\n";
	}
	print IMAGE "</td>\n";
	
	print IMAGE "</tr>\n";
	
	print IMAGE "</table>\n";
	
	# Now the prev / next bit
	print IMAGE "<hr>\n";
	print IMAGE "<center>\n";
	if ($i > 0) {
	    my $prev_page = basename($images[$i - 1]) . ".html";
	    print IMAGE "<a href=\"$prev_page\">prev</a>";
	} else {
	    print IMAGE "(first)";
	}

	print IMAGE " - <a href=\"index.html\">index</a> -  ";

	if ($i < $#images ) {
	    my $next_page = basename($images[$i + 1]) . ".html";
	    print IMAGE "<a href=\"$next_page\">next</a>";
	} else {
	    print IMAGE "(the end)\n";
	}
	print IMAGE "<p>\n";
	print IMAGE "$i of ", $#images;
	print IMAGE "</center>\n";

	print IMAGE $cgi->end_html;
	close IMAGE;
    }
}

sub check_prerequistes
{
    # Crude check to see if ImageMagick's convert program is runnable
    `convert`;
    if ($?) {
	print STDERR "Please install ImageMagick - this script requires the \"convert\" program\n";
	exit(1);
    }
}

sub parse_spec_file
{
    print STDERR "Image gallery generator from www.nobugs.org\n\n";
    
    open SPEC, "spec.txt" or usage();
    print STDERR " - Main images are $big_image_geometry\n";
    print STDERR " - Thumbnails will be $thumbnail_geometry.\n";
    print STDERR " - Writing self-contained gallery to \"$opt_output_dir\" subdirectory\n\n";
    
    my $current_image;
    while (<SPEC>) {
	if (/Title=(.*)/) {
	    $title = $1;
	    next;
	}
	if (/Date=(.*)/) {
	    $date = $1;
	    next;
	}
	
	if (/File=(.*)$/) {
	    $current_image = $1;
	    push @images, $1;
	    next;
	}
	die "Missing File= " unless $current_image;
	
	if (/Short=(.*)/) {
	    die "Duplicate Short=" if exists $short{$current_image};
	    $short{$current_image} = $1;
	    next;
	}
	
	if (/^$/) {
	    $long{$current_image} .= "<p>";
	    next;
	} else {
	    $long{$current_image} .= $_;
	    next;
	}
    }
}

###################################################
# The main bit

check_prerequistes();
parse_spec_file();

if (! -d $opt_output_dir) {
    mkdir($opt_output_dir) or die "Failed to create subdirectory \"$opt_output_dir\"";
}

build_index();

if (not $opt_indexonly) {
    build_pages();
}

print STDERR "\nYour new gallery is in the \"$opt_output_dir\" subdirectory .. enjoy!\n";
print STDERR "\nPlease give feedback via the www.nobugs.org website.\n";
